####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Bio.Types.PWM  
# DESC PSFM Object
#                                                                              #
################################################################################
"""
:NAME:
Bio.pwm

:DESC:
PWM Objects

:LONG_DESC:
    position weight matrices
    
classes:
    PWM
    
methods:

"""

# IMPORTS
#-------------------------------------------------------------------------------
from numpy import *
#-------------------------------------------------------------------------------
# IMPORTS


# OBJECTS
#-------------------------------------------------------------------------------

class PWM:
    """Position Weight Matrix
    
    """
    def __init__(self, psfm, p):
        """
        psfm -- Bio.Types.psfm.PSFM
        p    -- list of prob for A, C, G, T

        """
        self.psfm = psfm
        self.p = p
        m = psfm.matrix
        N = psfm.ba
        I,J = m.shape
        w = zeros( (I, 4), 'f')

        pseudoCount = N * 0.01

        for i in range(I):
            for j in range(4):
                w[i,j] = log( (m[i,j]*N + p[j] * pseudoCount) / (N + pseudoCount) / p[j] )

        self.w = w
        self.mu = self.mean()
        self.sigma = sqrt(self.var())

    def max(self):
        w = self.w
        I,J = w.shape
        S = 0.0
        for i in range(I):
            S += max([w[i,j] for j in range(4)])
        return S        

    def min(self):
        w = self.w
        I,J = w.shape
        S = 0.0
        for i in range(I):
            S += min([w[i,j] for j in range(4)])
        return S        

    def mean(self):
        w = self.w
        I,J = w.shape
        S = 0.0
        for i in range(I):
            for j in range(4):
                S += self.p[j] * w[i,j]
        return S

    def var(self):
        w = self.w
        I, J = w.shape

        V = 0.0
        for i in range(I):
            S = 0.0 # E(X)
            D = 0.0 # E(X^2)
        
            for j in range(4):
                S += self.p[j] * w[i,j]
                D += self.p[j] * w[i,j]**2
            V += D - S**2
        return V


#-------------------------------------------------------------------------------
# OBJECTS

#-------------------------------------------------------------------------------
# CODE




