####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Persistence.db
# DESC DB Persistency
#                                                                              #
################################################################################
"""
:NAME:
Persistence.db

:DESC:
DB Persistency

:LONG_DESC:
#pg_restore -U admin -a -d biodb -t x dump
#BASE = 'biodb'
#USER = 'admin'

"""
# IMPORTS
#-------------------------------------------------------------------------------
import copy
import os
############
# SQLite   #
############
#import sqlite

############
#PostgreSQL#
############
#import pgdb
#import pyPgSQL.PgSQL as pgdb

#-------------------------------------------------------------------------------
# IMPORTS

# CODE
#-------------------------------------------------------------------------------

BASE='my.db'
class DB:
    """High level DB facilities
    """
    def __init__(self, database=BASE, user=None, password=None, host=None, BASE_TYPE='PostgreSQL'):
        self.BASE_TYPE = BASE_TYPE
        self.database = database
        self.user     = user

        if self.BASE_TYPE == 'SQLite':
            self.connect  = sqlite.connect(database)
        elif self.BASE_TYPE == 'PostgreSQL':
            self.connect  = pgdb.connect(database=database, user=user, password=password, host=host)

        self.cursor   = self.connect.cursor()


    def new_cursor(self):
        self.cursor = self.connect.cursor()

    def close_cursor(self):
        self.cursor.close()

    def put(self, table, fields, id=None):
        """
        INSERT or UPDATE A ROW
        table : table
        fields  : fields dictionary (ex: {'RefSeq': 000023'})
        id if field exist: try to update row with id value in fields dictionary
        """
        fields = copy.copy(fields)

        for key in fields:
            if fields[key] == None:
                fields[key] = 'null'
                continue

            if type(fields[key]) == str:
                fields[key] = fields[key].replace("'", "\\'")
                fields[key] = "'" + fields[key] + "'"
            else:
                fields[key] = str(fields[key])
        action = 'insert'
        if id != None:
            self.cursor.execute('SELECT * FROM %s WHERE %s=%s;' % (table, id, fields[id]))
            if self.cursor.fetchall() != []:
                action = 'update'

        if action =='update':
            value = fields[id]
            for key in fields:
                fields[key] = str(key) + '=' + str(fields[key])
                
            #print ( "UPDATE %s SET %s WHERE %s=%s;" % (table, ','.join(fields.values()), id, value ) )
            self.cursor.execute( "UPDATE %s SET %s WHERE %s=%s;" % (table, ','.join(fields.values()), id, value ) )
        else:
            self.cursor.execute( 'INSERT INTO %s (%s) VALUES (%s);' % (table, ','.join(fields.keys()), ','.join(fields.values())))

        #self.connect.commit()


    def get(self, table, fields, cond=None):
        """
        table : table
        fields: list (ex: ['RefSeq'])
        cond  : dict (ex {RefSeq : 45454}) 
        RETURN a list dictionary for each found row (ex [{'RefSeq': 45454}])
        """
        if type(fields) == str:
            fields = [fields]

        if cond:
            #l1 = [ "%s=%s"   for key,value in cond.items() if type(value) == str]
            #l2 = [ "%s='%s'" for key,value in cond.items() if type(value) != str]

            l = []
            for key,value in cond.items():
                if value == None:
                    pass
                elif type(value) == str:
                    l.append("%s = '%s'" % (key, value))
                else:
                    l.append("%s = %s" % (key, value))

            cond = 'WHERE %s' % (' AND '.join(l))
        else:
            cond = ''
            
        query = 'SELECT %s FROM %s %s;' % (','.join(fields), table, cond)
        self.cursor.execute(query)
        res = self.cursor.fetchall()

        l = []
        for rec in res:
            d = {}
            for i in range(len(fields)):
                if type(rec[i]) == str:
                    d[fields[i]] = rec[i].replace('\\012', '\n')
                else:
                    d[fields[i]] =rec[i]
            l.append(d)

        return l


    def commit(self):
        self.connect.commit()


    def execute_file(self, filename):
        """Execute SQL command from file
        filename -- name of file with SQL commands
        """
        str = open(filename).read()
        self.cursor.execute(str)
        self.connect.commit()
        
    def __del__(self):
        self.connect.commit()
        self.cursor.close()



#-------------------------------------------------------------------------------
# CODE
