####################
# version 2.0
####################


"""
indexer draft

Notations:

Object: 0
properties of Objects: a,b,c...

N Objects: 0, 1, 2 ..., N-1

(a0, b0, c0)
(a1, b1, c1)

list of a values
a_ [ ]

a=x -> list of Object that have a=x


function that give attribute of an object: f(O) -> (a, b, c)

"""

#from set import Set
#from sets import Set
import dict

def group_objects(l, f):
    """
    l -- iterable bag of objects (list, ...)
    f -- function that take an object an return its group
    
    return dict where keys are group and value list of object in that group
    """
    d = {}
    for o in l:
        d.setdefault( f(o), []).append( o )
    return d


class SimpleIndex:
    """In memory index


    object are indexed by a tuple of attributes

    Example:
    Objets: lists of int

    use to index list porperties: len and max, min
    f = lambda o: (len(o), max(o), min(o))
    i = indexer.Index( ('len', 'max', 'min'))

    o1 = (1,2)
    o2 = (4,5,6)
    o3 = (4,5)
    
    i.extend( [o1,o2, o3] )
    i.build(f)
    print i.get((2,5,4))


    """

    def __init__(self, attributesNames, f=None):
        """
        function that take an object an return a tuple of is attributes        
        f -- indexing function
        """
        self.attributesNames = attributesNames
        self.objects = set()

    def add(self, o):
        self.objects.add(o)

    def add_list(self, ol):
        self.objects.update(ol)

    def get(self, key):
        """
        key: tuple (v1, v2, v3, ...)
        v1 corresponding to attribute1 ...

        return Set of objects
        """
        return self.idx[key]


    def build(self, f):
        """
        f -- indexing function
        function that take an object an return a tuple of is attributes
        """
        # init idx
        v = {}
        idx = {}
	
        # fill idx
        for o in self.objects:
            values, names = f(o), self.attributesNames
	    
            key = tuple(values)
            idx.setdefault(key, set()).add(o)
            
            for i in range(len(names)):
                name = names[i]
                value = values[i]
                #print name, value

                v.setdefault(name, set()).add(value)


        self.v = v 
        self.idx = idx



    def list_values(self, name, cond=None):
        """return list of value for attribut name
        optional condition restriction
        cond tuple (attname, value)
        """
        if cond == None:
            return self.v.get(name, [])
        else:
            att, value = cond
            i = list(self.attributesNames).index(name)
            icond = list(self.attributesNames).index(att)
            r = set()
            for k in self.idx:
                if k[icond] == value:
                    r.add(k[i])
            return list(r)
                
    


class Index:
    """In memory index

    Example:
    Objets: lists of int

    use to index list porperties: len and max, min
    f = lambda o: (len(o), max(o), min(o))
    i = indexer.Index( ('len', 'max', 'min'))

    o1 = (1,2)
    o2 = (4,5,6)
    o3 = (4,5)
    
    i.extend( [o1,o2, o3] )
    i.build(f)
    print i.get_or(len=2)


    """

    def __init__(self, attributesNames):
        """

        """
        self.attributesNames = attributesNames
        self.objects = set()

    def add(self, o):
        self.objects.add(o)

    def add_list(self, ol):
        self.objects.update(ol)

    def get_OR(self, **kwargs):
        """
        get(attname1=val, attname2=val,...)
        return list of object matching at least to one the given attributes values
        """

        s = set()

        for k,v in kwargs.items():
            if self.idx.has_key(k):
                s.update(self.idx[k].get(v, []))
        return list(s)

    def get_AND(self, **kwargs):
        """
        get(attname1=val, attname2=val,...)
        return list of object matching all the given attributes values
        """

        first = True

        for k,v in kwargs.items():
            if not self.idx.has_key(k):
                continue
            s = set( self.idx[k].get(v, []) )
            if first:
                S = s
                first = False
                
            S = S & s


        return list(S)
                


    def build(self, f):
        """
        f -- indexing function
        function that take an object an return a tuple of is attributes
        """
        # init idx
        t = []
        v = {}
        idx = {}
        for name in self.attributesNames:
            idx[name] = {}

        #

        # fill idx
        for o in self.objects:
            values, names = f(o), self.attributesNames

            t.append( values )

            for i in range(len(names)):
                name = names[i]
                value = values[i]
                #print name, value

                v.setdefault(name, set()).add(value)
                idx[name].setdefault(value, set()).add(o)
        
        #
        #self.t = t
        self.v = v 
        self.idx = idx

    def list_values(self, name):
        return self.v.get(name, [])



    def extend(self, other):
        # objects
        self.objects.union_update(other.objects)

        # self.idx
        for name in self.idx:
            for k in self.idx[name].keys().extend(other[name].keys()):
                self.idx[name].setdefault(k, set()).union_upextend(other.idx[name].get(k, set()))


        # self.v
        for name in self.v:
            self.v[name].union_update(other.v[name])




def test():
    
    i = SimpleIndex( ('len', 'max', 'min') )
    f = lambda o: (len(o), max(o), min(o))
    o1 = (1,2)
    o2 = (4,5,6)
    o3 = (4,5)
    
    i.add_list( [o1,o2, o3] )
    i.build(f)

    #print i.get_and(len=2, min=4)
    print i.list_values('max', ('len', 2))

