####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Types.iter
# DESC Iterator utilities
#                                                                              #
################################################################################
"""
:NAME:
Types.iter

:DESC:
Iterator Utilities

"""

# IMPORTS
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# IMPORTS

# OBJECTS
#-------------------------------------------------------------------------------


########################################
#                                      #
#          Chunk                       #
#                                      #
########################################
def chunk(array, chunkSize=10):
    """Chunk a list in sub lists of length chunkSize
    chunkSize -- int
    """
    l = []
    for i in range(0, len(array) - chunkSize+1, chunkSize):
        l.append(array[i:i+chunkSize])
    if (len(array) % chunkSize) != 0:
        l.append(array[-(len(array) % chunkSize):])
    return l

def reversed(l):
    """Return a reversed iterator on list
    """
    if hasattr(l, 'keys'):
        raise ValueError("mappings do not support reverse iteration")
    i = len(l)
    while i > 0:
        i -= 1
        yield l[i]

class Chunk:
    """Chunk that work with Iterator
    """
    def __init__(self, iterator, chunkSize=10):
        """
        iterator  -- an iterable object
        chunkSize -- int (the chunk size)
        """
        self.iterator = iterator
        self.chunkSize = chunkSize
        self.end = 0

    def __iter__(self):
        return self
    
    def next(self):
        if self.end:
            raise StopIteration
        l = []
        try:
            for i in range(self.chunkSize):
                l += [self.iterator.next()]
        except StopIteration:
            if l == []:
                raise
            self.end = 1
        return l

#-------------------------------------------------------------------------------
# OBJECTS

