####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME UI.gtk  
# DESC GTK GUI
#                                                                              #
################################################################################
"""
:NAME:
UI.gtk

:DESC:
GTK GUI

:DOC:

Callback
--------
add a callback to an object
  - default object event, function with no args:
    object < function
  - default object event, function with args:
    object < (function, arg1, arg2, ...)
  - other event:
    object <= ('event_name', function, arg1, arg2, ...)


Packing     
-------
  -VBox/HBox
  -Table

Packing with VBox/HBox
~~~~~~~~~~~~~~~~~~~~~~
1) Window
  - window & object
2) VBox, HBox
  - VBox() + obj1 + obj2 + ....
Example:
  ------------------
  |                |
  |       OK       |
  |                |
  |----------------|
  |        |       |
  |   x    |   y   |
  |        |       |
  ------------------
v = VBox() + Label('OK') + ( HBox() + Label('x') + Label('y'))

Packing with Table
~~~~~~~~~~~~~~~~~~
1) Create a new empty table
t = Table(2,2) #two rows, two columns
t[0,0] = Label('a')
t[0,1] = Label('b')
t[1,0] = Label('c')
t[1,1] = Label('d')
  -------------------
  |        |        |
  |    a   |    b   |
  |        |        |
  -------------------
  |        |        |
  |    c   |    d   |
  |        |        |
  -------------------

2) With List
2.a) Same examle as above
t = Table(tab= [ [ Label('a'), Label('b') ],
             [ Label('c'), Label('d') ],
           ]
2.b) Table with one row (external[] can be omitted)
t = Table(tab= [ Label('a'), Label('b') ] )


"""
# IMPORTS
#-------------------------------------------------------------------------------
import pygtk
pygtk.require("2.0")
import gtk
import signal
#-------------------------------------------------------------------------------
# IMPORTS

# CODE
#-------------------------------------------------------------------------------
class Object:
    def __init__(self, gtkObject=None, hidden=False):
        self.gtkObject = gtkObject or self
        self.hidden = hidden
        if not hidden:
            self.show()

    ##
    #  Show/Hide
    ##
    def show(self, yes=True):
        if yes:
            self.hidden = False
            if hasattr(self.gtkObject, 'show_all'):
                self.gtkObject.show_all()
        else:
            self.hidden = True
            self.gtkObject.hide_all()            
            
    def hide(self):
        self.show(False)

    def update(self):
        if not self.hidden:
            self.show()

    def __del(self):
        gtk.gtk_widget_destroy(self.gtkObject)


    def size(self, width=None, height=None):
        self.gtkObject.set_size_request(width, height)

    def get(self):
        return self.gtkObject

    def get_gtkObject(self):
        return self.gtkObject


def gtk_object(object):
    """
    need work
    """
    if isinstance(object, Object):
        return object.get()
    else:
        return object



class Activable:
    def __init__(self, gtkObject=None, action=None, event='clicked'):
        """
        action: tuple (function , arg1, arg2,...) or function
       """
        self.event = event
        self.gtkObject = gtkObject or self
        self.set = 0
        if action:
            self.clicked(action)

    def activated(self, *args):
        if not args:
            return self
        self.__callback(self.event, *args)
        self.set = 1
        return self
        
    def callback(self, event, *args):
        if not args:
            return self
        self.__callback(event, *args)
        return self
        
        
    def __callback(self, event, function, *args):
        assert(callable(function))
        self.gtkObject.connect(event, lambda w, d: function(*args), None)


    def __lt__(self, args):
        """
       default callback
       args -- function or (function, arg1, ...)
       Example: object < hello
       """
        if type(args) == list or type(args) == tuple:
            return self.callback(self.event,*args)
        else:
            return self.callback(self.event, args)


    def __le__(self, args):
        """
       args -- (event, function, args)
       Example: object <= ('clicked', hello, name)
       """
        if type(args) == function:
            return self.activated()
        if not (type(args) == list or type(args) == tuple):
            print 'invalid callback'
        
        if type(args[0]) == str:
            return self.callback(*args)
        else:
            return self.callback(self.event, args)       




########################################
#                                      #
#            Packing                   #
#                                      #
########################################
class Box:
    """
    Abstract box

    """
    expand  = False
    fill    = False
    spacing = 3
    padding = 3

    def __init__(self, spacing=None, padding=None, expand=None, fill=None):
        self.spacing = spacing
        self.padding = padding
        self.expand  = expand
        self.fill    = fill
        if self.spacing == None:
            self.spacing = self.__class__.spacing
        if self.padding == None:
            self.padding = self.__class__.padding
        if self.expand == None:
            self.expand = self.__class__.expand
        if self.fill == None:
            self.fill = self.__class__.fill


    def append(self, object, padding=None):
        if padding == None:
            padding = self.padding
        object = gtk_object(object)
        self.gtk.pack_start(object, expand=self.expand, fill=self.fill, padding=padding)
        self.update()
        return self

    def __add__(self, object):
        return self.append(object)

    def __and__(self, object):
        return self.append(object)


class HBox(Object, Box):
    def __init__(self, spacing=3, padding=3, expand=False, fill=False):
        Box.__init__(self, spacing=spacing, padding=padding, expand=expand, fill=fill)
        self.gtk = gtk.HBox(homogeneous=False, spacing=spacing)
        Object.__init__(self, self.gtk)

class VBox(Object, Box):
    def __init__(self, spacing=3, padding=3, expand=False, fill=False):
        Box.__init__(self, spacing=spacing, padding=padding, expand=expand, fill=fill)
        self.gtk = gtk.VBox(homogeneous=False, spacing=spacing)
        Object.__init__(self, self.gtk)


class Table(Object):
    expand  = False
    padding = 3
    def __init__(self, rows=None, cols=None, expand=None, spacing=None, padding=None, tab=None):
        const = 's'

        self.tab     = tab
        self.expand  = expand 
        self.padding = padding

        if self.padding == None:
            self.padding = self.__class__.padding

        if self.expand == None:
            self.expand = self.__class__.expand


        spacing = False
        self.rows, self.cols = rows, cols
        if tab != None:
            if type(tab[0]) == list or type(tab[0]) == tuple:
                self.rows = len(tab)
                self.cols = len(tab[0])
                const = 'n'
            elif type(tab) == list or type(tab) == tuple:
                self.cols = len(tab)
                self.rows = 1
                const = 'r'

        self.table = gtk.Table(rows=self.rows, columns=self.cols, homogeneous=False)
        Object.__init__(self, self.table)


        self.table.set_row_spacings(spacing)
        self.table.set_col_spacings(spacing)


        if const == 'r':
            for j in range(len(tab)):
                self.attach(tab[j], (0,j))
        elif const == 'n':
            for i in range(len(tab)):
                for j in range(len(tab[i])):
                    self.attach(tab[i][j], (i,j))          

    def get_size(self):
        return (self.rows, self.cols)


    def attach(self, object, loc=(0,0)):
        if self.expand:
            self.xyoptions = gtk.EXPAND|gtk.FILL
        else:
            self.xyoptions = gtk.FILL

        if isinstance(object, Object):
            object = object.get()
        self.table.attach(object, loc[1], loc[1]+1, loc[0], loc[0]+1,
             xoptions=self.xyoptions, yoptions=self.xyoptions, xpadding=self.padding, ypadding=self.padding)
        self.update()

    def __setitem__(self, loc, object):
        self.attach(object, loc)

########################################
#                                      #
#         Windows & Dialog             #
#                                      #
########################################

class Window(Object):
    def __init__(self, width=600, height=400, object=None, hidden=False):
        type = gtk.WINDOW_TOPLEVEL
        self.window = gtk.Window(type)
        self.window.set_default_size(width, height)
        self.window.connect("delete_event", self.__delete_event)
        self.window.connect("destroy", self.__destroy)
        Object.__init__(self, self.window, hidden=hidden)
        if object != None:
            self.append(object)


    def __delete_event(self, widget, event, data=None):
        #return FALSE emit destroy signal
        return False

    def __destroy(self, widget, data=None):
        gtk.main_quit()

    def append(self, object):
        self.child = object
        if isinstance(object, Object):
            object = object.get()
        self.window.add(object)
        return self
        
    def __and__(self, object):
        return self.append(object)    

    def __add__(self, object):
        return self.append(object)  


class Dialog(Object, Activable):
    def __init__(self, title='dialog'):
        self.gtk = gtk.Dialog(title, None,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            ('OK', gtk.RESPONSE_ACCEPT, 'Cancel', gtk.RESPONSE_REJECT))

        self.gtk.set_has_separator(False)
        Object.__init__(self, self, hidden=1)
        Activable.__init__(self) 
        self.defaultCallback = 1

    def callback(self, event, *args):
        if not args:
            return self
        self.function  = args[0]
        self.args      = args[1:]
        self.defaultCallback = 0

        return self
        

    def run(self):
        if self.gtk.run() == gtk.RESPONSE_ACCEPT:
            if not self.defaultCallback:
                print 'call %s with args [%s]' % (`self.function`, `self.args`)
                self.function(*self.args)
            self.gtk.destroy()
            return 1
        else:
            self.gtk.destroy()
            return 0

    def append(self, object):
        v = HBox(expand=1, padding=12) + (VBox(padding=12) + object)
        self.gtk.vbox.pack_start(v.get_gtkObject(), True, True, 0)
        return self

    def __and__(self, object):
        return self.append(object)

    def __add__(self, object):
        return self.append(object)


class Message(Object):
    def __init__(self, msg='message', title='info', wait=False):

        self.gtk = gtk.Dialog(title, None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)

        self.gtk.set_has_separator(False)
        Object.__init__(self, self, hidden=1)

        self.gtk.set_default_size(300,80)
        self.label = Label('%s' % msg)

        v = VBox() + ( VBox(padding=12) + (HBox(padding=12, expand=1) + self.label) )

        b = VBox() + Button('OK', self.gtk.response, gtk.RESPONSE_OK)
        self.gtk.action_area.set_layout(1)
        self.gtk.action_area.pack_start(b.get_gtkObject(), True, True, 0)

        self.gtk.vbox.pack_start(v.get_gtkObject(), True, True, 0)

        if self.gtk.run():
            self.gtk.destroy()




########################################
#                                      #
#            Buttons                   #
#                                      #
########################################

class Button(Object, Activable, gtk.Button):
    def __init__(self, txt='OK', *args):
        gtk.Button.__init__(self, txt)
        Activable.__init__(self, event='clicked')        
        Object.__init__(self)
        self.activated(*args)
        self.size(64,32)


class SpinButton(Object, gtk.SpinButton):
    def __init__(self, vrange=(0.0,1.0), step=None):
        step = step or float(vrange[1] - vrange[0]) / 100.0
        if type(step) == float:
            digits = 2
        else:
            digits = 0            
        adjustment = gtk.Adjustment(value=vrange[0], lower=vrange[0], upper=vrange[1], step_incr=step, page_incr=0, page_size=0)
        gtk.SpinButton.__init__(self, adjustment=adjustment, climb_rate=step, digits=digits)
        Object.__init__(self, self)

class CheckButton(Object, Activable):
    def __init__(self, txt='check button'):
        self.gtk = gtk.CheckButton(txt)
        Activable.__init__(self, self)
        Object.__init__(self, self.gtk)

    def __getitem__(self, checked=None):
        if checked == None:
            if self.gtk.get_active():
                return True
            else:
                return False
        elif checked:
            self.gtk.set_active(True)
        else:
            self.gtk.set_active(False)            


########################################
#                                      #
#           Label & Text               #
#                                      #
########################################

class Label(Object, gtk.Label):
    def __init__(self, txt='label'):
        gtk.Label.__init__(self)
        self.set_markup(txt)
        Object.__init__(self, self)

    def set_text(self, txt):
        self.set_markup(txt)

    def __call__(self, txt):
        self.set_text(txt)


class TextEntry(Object):
    def __init__(self, txt='', visible=1):
        self.gtk = gtk.Entry()
        Object.__init__(self, self.gtk)
        self.gtk.set_text(txt)
        self.gtk.set_visibility(visible)

    def __call__(self, txt=None):
        if txt == None:
            return self.gtk.get_text()
        else:
            return self.gtk.set_text(txt)

    def get_text(self):
        return self.gtk.get_text()


class Combo(Object, gtk.Combo):
    def __init__(self, list=('a', 'b')):
        gtk.Combo.__init__(self)
        Object.__init__(self, self)
        #self.set_popdown_strings(list)
        #self.set_use_arrows_always(True)


class Image(Object):
    def __init__(self, file=None):
        self.gtk = gtk.Image()
        Object.__init__(self, self.gtk)

        if file:
            self.gtk.set_from_file(file)


class Menu(Object, Activable):
    def __init__(self, list=('a', 'b')):
        self.menu = gtk.Menu()
        self.opt  = gtk.OptionMenu()
        for i in list:
            item = gtk.MenuItem(i)
            item.set_name(i)
            self.menu.append(item)
        self.opt.set_menu(self.menu)
        Activable.__init__(self, self.opt, event='changed')
        Object.__init__(self, self.opt)

    def get_selected(self):
        return self.menu.get_active().get_name()

        
class Gui:
    def __init__(self):
        gtk.threads_init()
        signal.signal (signal.SIGINT, signal.SIG_DFL) #be able to CTR-C to kill app


    def __del(self):
        pass

    def run(self):
        gtk.threads_enter()
        gtk.main()
        gtk.threads_leave()


    def __del__(self):
        pass
        #self.run()

    def quit(self):
        gtk.main_quit()

#-------------------------------------------------------------------------------
# CODE


def test3():
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    box1 = gtk.VBox(False, 0)


    box1.pack_start(gtk.Button('1'), False, False, 0)
    box1.pack_start(gtk.Button('1'), False, False, 0)

    quitbox = gtk.HBox(False, 0)
    button = gtk.Button("Quit")
    quitbox.pack_start(button, True, False, 0)
    box1.pack_start(quitbox, False, False, 0)


    window.add(box1)
    window.show_all()
    gtk.main()

if __name__ == '__main__':
    test3()
