####################
# version 2.0
####################


import numpy


def count(values, xrange):
    """
    count for each x in xrange number of values == to x

    return count list
    """
    d = [0]*len(xrange)
    for v in values:
        if v >= xrange[0] and v <= xrange[1]:
            d[v] += 1
    return d

def hist(values, bins, leftbound=1):
    """
    values -- list of values
    bins   -- list of boundaries

    X: number of values
    return tuple(bins, count)
    return list -inf < X < bins[0] <= X < bins[1] ... < +inf
    [-inf,bins[0][ .. [bins[0],bins[1][ ..
    """
    values = numpy.array(values).astype('f')
    bins = numpy.array(bins).astype('f')
    if not leftbound:
        values = values * -1.0
        bins = bins[::-1] * -1.0

    #print values, bins
    n = numpy.searchsorted(numpy.sort(values), bins)
    n = numpy.concatenate([n, [len(values)]])
    l = n.tolist()
    for i in range(1, len(n)):
        l[i] = n[i] - n[i-1]

    l = numpy.array(l)
    if not leftbound:
        l = l[::-1]
        bins = bins[::-1] * -1.0

    return bins.tolist(), l.tolist()


def freq(count):
    """Compute frequency

    count -- list of count
    
    """
    a = numpy.array(count)
    #print a, Numeric.sum(a)
    a = a / float(numpy.sum(a))
    #print a
    return a.tolist()



def reverse_cumsum(l):
    """
    l -- list 
    cumulative sum (from tail to head)

    """
    
    l = numpy.cumsum(l[::-1])
    return l[::-1]



def cum_pack_values(values, pack={}, xrange=(-10,10), step=0.5, nmin=5):
    """Construct a cumulative count histogram
    values  -- list of values
    pack    -- dict keys=x (value) value=list of number of item with value <=x
    xrange  -- boundaries
    step    -- step
    nmin    -- min bin size
    
    """
    epsilon = 0.000001

    evrange = numpy.arange(xrange[0], xrange[1], step)
    l = numpy.searchsorted(numpy.sort(values), numpy.arange(xrange[0], xrange[1], step+epsilon))

    for i in range(len(evrange)):
        if l[i] < nmin:
            continue
        if len(values) - l[i] < nmin:
            break

        pack[evrange[i]] = pack.get(evrange[i], [])
        pack[evrange[i]].append(l[i])
