####################
# version 2.0
####################




"""
 outputXML.py
 
 Toolbox to create result files from tfme: 
 	- one xml with the extracted windows
	- one csv with the correlations between windows
	- one csv with the hits of all the matrices in the windows
 
 author: Laurie Tonon
 date: 5/11/09
 
 methods:
    printxml
    format
    indent
"""

import xml.etree.cElementTree as etree
import sys
import os
import Core.UI.cli as cli
import random


def printxml(r,corr,output,hitsToSave):
    """ 
	Create three result files: one with the extracted windows from TFME another other with the correlations 2 by 2 of these windows
	and a last one with the hits found
	
	r	    -- results of tfme= details of the windows
	
	corr	-- correlations 
	
	output 	-- output directory where to put the xml files
    """

    
    #available colors for drawing when the result file is read
    colors=['blue', 'pink','grey','yellowgreen','gold', 'magenta','green','red','mediumaquamarine','brown', 'orange', 'purple', 'turquoise',  'chartreuse', 'tomato', 'navajowhite', 'cornflowerblue','rosybrown','fuchsia','palevioletred', 'maroon', 'olive','aqua','darkorange',  'burlywood',  'darkorchid','lightgrey',  'lightblue',  'lightsalmon',  'dodgerblue', 'peru', 'darkslategrey', 'forestgreen', 'darkgrey', 'paleturquoise',  'cyan', 'mediumorchid', 'skyblue',  'darkturquoise',  'darkseagreen', 'darkviolet', 'saddlebrown',  'darkslateblue', 'lightskyblue',  'mediumvioletred', 'springgreen', 'limegreen', 'darkmagenta', 'plum',  'mediumspringgreen', 'darkgoldenrod','dimgray', 'sandybrown',  'lightgreen', 'indigo', 'lime', 'mediumturquoise', 'dimgrey', 'tan',  'olivedrab',   'darkolivegreen', 'slateblue', 'darkblue', 'seagreen', 'royalblue', 'darkcyan', 'lightseagreen',  'deepskyblue', 'salmon',  'darkred', 'indianred', 'steelblue', 'firebrick',  'aquamarine','hotpink', 'orchid',  'mediumseagreen',  'cadetblue', 'palegreen',  'mediumslateblue','crimson', 'violet', 'darkgreen','deeppink','blueviolet' ]

    try:
    
        #***************************************************
        # create XML file with clusters
        #**************************************************
        results = etree.Element("results")

        colormap={}
	
	    # create a new tag for each key of the results
        for i in r.keys():
		
	    	# print the info common to all windows
		
            if i == 'parameters':
                child = etree.SubElement(results, i)
                for j in r[i].keys():
                    child2=etree.SubElement(child, j)
                    child2.text=str(r[i][j])

		    # expend the info for each window
            elif i== 'R':
                child = etree.SubElement(results, i)
                strike=0
                for j in range(len(r[i])):
				
                    child3=etree.SubElement(child, "result")

                    if r[i][j]['matrixId'] in colormap.keys():
                        childc=etree.SubElement(child3,"color")
                        childc.text=colormap[r[i][j]['matrixId']]
                    else:  
                        childc=etree.SubElement(child3,"color")
                        childc.text=colors[strike]
                        colormap[r[i][j]['matrixId']]=colors[strike]
                        if strike < len(colors)-1:
                            strike +=1
                        else:
                            strike=0                        

                    for k in r[i][j].keys():
					
                        tagsToSave=['i','j','matrixId','matrixName','matrixInfo','matrixGC','matrixLen','pvalue','sequencesGC']
                        if k in tagsToSave:
                            child4=etree.SubElement(child3, k)
                            child4.text=str(r[i][j][k])
                        if k == 'score':
                            child4=etree.SubElement(child3, k)
                            child4.text=str(map(format,r[i][j][k]))    
                        if k == 'hitsFull':
                            hits=r[i][j][k][1:]
                            child4=etree.SubElement(child3, k)
                            for h in hits:
                                child5=etree.SubElement(child4,"hit")
                                child6=etree.SubElement(child5,"seqID")
                                child6.text=(h.split()[0])
                                child6=etree.SubElement(child5,"posRel")
                                child6.text=(h.split()[1])
                                child6=etree.SubElement(child5,"posAbs")
                                child6.text=(h.split()[2])
                                child6=etree.SubElement(child5,"strand")
                                child6.text=(h.split()[3])
                                child6=etree.SubElement(child5,"score")
                                child6.text= h.split()[4]
                                child6=etree.SubElement(child5,"sequence")
                                child6.text=(h.split()[5])

                    
            elif i == 'matricesSize':
                child = etree.SubElement(results, "nbOfMatrices")			
                child.text=str(r[i])
            elif i == 'sequencesSize':
                child = etree.SubElement(results, "nbOfSequences")			
                child.text=str(r[i])
            else:
                child = etree.SubElement(results, i)			
                child.text=str(r[i])
	
	    ###########################
	    # save xml file
	    ###########################
        indent(results)
        tree=etree.ElementTree(results)
        tree.write(os.path.join(output,"windows.xml"), encoding="utf-8")


        #*******************************************
        # Create CSV file with correlations 
        #*******************************************
        correlations=open(os.path.join(output,"correlations.csv"),'w')
        if corr:
            for t in corr[0].keys():
                correlations.write(t)
                correlations.write(';')
            correlations.write("\n")
            for c in range(len(corr)):
    		
                for k in corr[c].keys():
    			
                    correlations.write(str(corr[c][k]))
                    correlations.write(';')
                correlations.write("\n")
    
    except IOError:
        cli.error("An error occured in creating the output files")
        sys.exit(1)


    #******************************
    # Create CSV file with hits
    #******************************
    try:
        
        pwms=[n['matrixId'] for n in r['R']]
        hitfile=open(os.path.join(output,"hits.csv"),"w")
        hitfile.write("matrixID;sequenceID;positionAbs;strand;size;sequence\n")

        random.shuffle(colors)
        strike=0

        for k in hitsToSave:
            for i in k:
                
                # set a unique color to each matrix (used in the vizualisation of hits in a window)
                if k[0][0] in pwms:
                    color=colormap[k[0][0]]
                else:
                    if(k[0][0] in colormap.keys()):
                        color=colormap[k[0][0]]
                    else:
                        color=colors[strike]
                        colormap[k[0][0]]=color
                        if strike<(len(colors)-1):
                            strike+=1
                        else:
                            strike=0
                # write the hit details in the csv file
                for j in i:
                    hitfile.write(str(j))
                    hitfile.write(";")

            hitfile.write(color)
            hitfile.write("\n")
           
    except IOError:
		cli.error("An error occured in creating the output files")
		sys.exit(1)


def format(score):
    """
    format a score with 3 decimals
    
    score  -- a float value
    
    return -- a float value with 3 decimals
    """	
    return "%.3f" % score


def indent(elem, level=0):
    """
    process an elementtree to put XML indentation
    
    elem -- an element of elementtree
    
    level -- the level of the sarting point of the elementtree (0 = root)
    
    """
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i
