####################
# version 2.0
####################


"""
Tab=(position, strand, score, log_P-value)
ex: (10, '+', 10.6, -3.0)

class:
    Tab
    
methods:
    location2indicies
    location2slice
    indicies2location
    
"""
from numpy import *

import Core.Types.tab
import Core.UI.cli as cli

import Bio.sequence
import Bio.hit

def location2indicies(location, genelocation):
    """Convert location to list indicies

    location     -- feature location in REF coordinate
    genelocation -- gene location in REF coordinate
    """
    return (location[0] - genelocation[0], location[1] - genelocation[0]+1)


def location2slice(location, genelocation):
    """Convert location to list indicies

    location     -- feature location in REF coordinate
    genelocation -- gene location in REF coordinate
    """
    return slice(location[0] - genelocation[0], location[1] - genelocation[0]+1)


def indicies2location(indicies, genelocation):
    """Convert list indicies to location

    genelocation -- gene location in REF coordinate
    """
    return (genelocation[0] + indicies[0], genelocation[0] + indicies[1]-1)

class Tab:
    """
    Store k and s Tabs
    """

    def __init__(self, sequences):
        self.sequences = sequences
        self.location = Bio.sequence.find_location(sequences)
        self.size = len(sequences)
        
        L = self.location[1] - self.location[0] + 1
        self.T_k = zeros( ( self.size, L ), 'i')
        self.T_s = zeros( ( self.size, L ), 'i')

        self.T_alpha_b = zeros( ( self.size, L ), 'f')
        self.T_alpha   = zeros( ( self.size, L ), 'f')

    def build_T_k(self, hits):
        """
        hits -- hits for 1 matrix
        """
        for i in range(self.size):
            self.T_k[i,:] += Bio.hit.expand(hits[i], self.sequences[i].location, self.location)

    def build_T_s(self):
        for i in range(self.size):
            a,b = location2indicies(self.location, self.sequences[i].location)
            #compatibility with Numeric 23.7
            I = self.T_s[i]
            put(I, range(a,b), I+1)
            self.T_s[i] = I
            #
            #self.T_s[i,a:b] += 1

            #
    def build_T_alpha_b(self, bgModels):
        """
        bgModels -- list of bgModel corresponding to sequences in t
        """
        assert(len(self.sequences) == len(bgModels))
        for i in range(len(bgModels)):

            I = self.sequences[i].locationTSS[0] - bgModels[i].location[0]
            self.T_alpha_b[i,:] += bgModels[i].p[I:I+len(self.T_alpha_b[i,:])]


    def build(self, hits, bgModels, parameters):
        """
        hits     -- hits for 1 matrix (list of list of tuple)        
        bgModels -- list of bgModel corresponding to sequences in t
        """
        self.build_T_k(hits)
        self.build_T_s()
        self.build_T_alpha_b(bgModels)
        self.T_alpha = self.T_alpha_b * float(parameters['ratio'])

            






