#! /usr/bin/env python

###################
# version 2.0
###################

#############################################################################################
# fasta2dump
# Program that convert wight matrices into a serialized object saved into a binary file. 
# The matrices can be weight matrices from public Transfac, from Jaspar, or personal 
# text files in Jaspar format.
#
# author: Laurie Tonon
# date: 28/10/2009
#
#############################################################################################

import os, sys
PYTHONPATH = [['TFME'], ['TFME', 'MLib.zlib']]
if sys.version_info < (2, 3): 
    print >> sys.stderr, 'python >= 2.3 is required!'; sys.exit()
[sys.path.insert(1, os.path.join(sys.path[0], *path)) for path in PYTHONPATH]


import imp

#********************************************************************
# Check dependencies
#********************************************************************
try:
    import numpy
    version= numpy.__version__

except ImportError:
    print 'WARNING: TFM-Explorer requires Numpy '
    sys.exit(1)
    
try:
    import Pyrex

except ImportError:
    print 'WARNING: TFM-Explorer requires Pyrex '
    sys.exit(1)



import optparse

import Bio.psfm
import Bio.sequence
import Core.UI.cli as cli 

usage = """
     python   %prog --input=filename --output=filename --type=TYPE

        TYPE can be on the following:
          *) jaspar (JASPAR matrices dump)
          input corresponds to the JASPAR directory
          Example %prog --input=~/Data/JASPAR --output=jaspar.mat.db --type=jaspar

          *) transfac
          input corresponds to transfac matrix data in .dat file format
          Example %prog --input=transfac.dat --output=transfac.mat.db --type=transfac

          *) perso (personnal matrices)
          input corresponds to a txt file in a fixed format (see documentation for details about the format)
          Example %prog --input=perso.txt --output=perso.mat.db --type=perso
"""

parser = optparse.OptionParser(usage)

parser.add_option("-t", "--type", dest="type", help="input data type (jaspar, transfac or perso)", action="store", default=None)
parser.add_option("-i", "--input", dest="input", help="input filename", action="store", default=sys.stdin)
parser.add_option("-o", "--output", dest="output", help="output filename", action="store", default="out.mat.db")
parser.add_option("-v", "--verbosity", action="store", dest="verbosity", type="int", default=1, help="display complete process on screen (between 1 and 3) (default=1)")
parser.add_option("--debug", dest="debug", action="store_true", default=False, help="show program errors. For developpers only")

(options, args) = parser.parse_args()
cli.VERBOSITY = options.verbosity

def run(args, options):

    #****************************************
    # Check user inputs
    #****************************************
    if not options.output:
        cli.error("No outout file provided. Cannot continue.")
        sys.exit(1)
    elif os.path.dirname(options.output) != '' and not os.path.isdir(os.path.dirname(options.output)): # check if the output directory exists
        cli.error('The output directory does not exists')
        sys.exit(1)
    else:    
        if os.path.isfile(options.output):
            try:
                os.remove(options.output)
            except OSError:
                cli.error("A dump file with the same name as "+options.output+" already exists and cannot be removed. Please delete or rename this file and lauch matrix2dump.py again.")
                sys.exit(1)
        elif os.path.isfile(options.output+'.db'):
            try:
                os.remove(options.output+'.db')
            except OSError:
                cli.error("A dump file with the same name as "+options.output+'.db'+" already exists and cannot be removed. Please delete or rename this file and lauch fasta2dump.py again.")
                sys.exit(1)
    
    if not options.input:
        cli.error("No input file provided. Cannot continue.")
        sys.exit(1)
        
    if options.type == 'jaspar':
        #input : JASPAR_CORE directory
        if not os.path.isdir(options.input):
            cli.error("The Jaspar directory cannot be found")
            sys.exit(1)
        else:
            Bio.psfm.jaspar2dump(options.input, options.output)

    elif options.type == 'transfac':
        if not os.path.isfile(options.input):
            cli.error("The transfac file cannot be found")
            sys.exit(1)
        else:
            Bio.psfm.transfac2dump(options.input, options.output)

    elif options.type == "perso":
        if not os.path.isfile(options.input):
            cli.error("The text file cannot be found")
            sys.exit(1)
        else:
	        Bio.psfm.perso2dump(options.input,options.output)

    else:
        cli.error('unreconized input type')
        sys.exit(1)

    print "TFM-Explorer matrix to dump converter job succesfully ended" 
if __name__ == '__main__':
    try:
        run(args, options)
    except:
        cli.error('Error while running matrix2dump')
        if options.debug:
            raise
